require(stringr)
source('../Rhelpers/TexregRor.R')

# Subset data for estimation
# -------------------------------------------------------------------------
MySubset <- function(..., data, vars.subs = c(), dep.var = c()) {

  vars <- DropMinusVars(c(..., vars.subs, dep.var))
  vars.no.int <- DropInteractions(vars)

  df <- data[complete.cases(data[, c(vars.no.int)]), ]

  as.data.frame(df)
}

# Transform dataset for Bell-Jones estimation
BjTransform <- function(df, terms) {
  for (term in terms) {
    df[[paste0(term, '.btw')]] <- mean(df[[term]])
    df[[term]]                 <- center(df[[term]])
  }
  df
}

# Add between variables and interactions to vector of terms for Bell-Jones
# estimation
BjT <- function(..., re.terms = t.re) {
  newt <- ApplyToPermutations(c(...), '*', function(x) paste0(x, '.btw')) %>%
    setNames(NULL) %>%
    sort() %>%
    c(re.terms)
}

DropSparseEntities <- function(df, entity, target.dim, required.ratio = 1,
                               vars.complete = 'ALL',
                               check.numeric.entities = TRUE) {

  # We want to consider complete cases with all necessary variables
  if (vars.complete[1] == 'ALL') {
    vars.complete <- names(df)
  }
  df.restr <- df[, unique(c(target.dim, entity, vars.complete))]
  df.restr <- df.restr[complete.cases(df.restr), ]

  max.nobs <- length(unique(df.restr[, target.dim]))
  nobs.per.entity <- table(df.restr[, entity])
  if (required.ratio <= 1) {
    keep.entity <- nobs.per.entity / max.nobs >= required.ratio
  } else {
    keep.entity <- nobs.per.entity >= required.ratio
  }
  entities.to.keep <- names(nobs.per.entity)[keep.entity]
  new.df <- droplevels(df[df[, entity] %in% entities.to.keep, ])

  # Print statistics
  cat("Dropping sparse entities from id:", entity, fill = TRUE)
  cat("Keeping entities with at least",
      ceiling(max.nobs * required.ratio), "of a maximum of", max.nobs,
      "observations.", fill = TRUE)
  cat("Keeping", sum(keep.entity), "of", length(nobs.per.entity),
      "entities.", fill = TRUE)
  cat("Keeping", nrow(new.df), "of", nrow(df), "observations.", fill = TRUE)

  entities.to.drop <- names(nobs.per.entity)[!keep.entity]
  if (length(entities.to.drop) %in% 1:30) {
    cat("Here is a list of dropped entities and their corresponding numbers",
        "of observations:", fill = TRUE)
    tmp <- data.frame(entities.to.drop, nobs.per.entity[!keep.entity])
    names(tmp) <- c(entity, 'NOBS')
    rownames(tmp) <- NULL
    print(tmp)
  }

  if (check.numeric.entities) {
    # If our entities of interest are numeric values (i.e. years) the user may be
    # surprised if we don't drop values at the edges, but from the middle instead.
    # We should let him know.
    entity.num <- suppressWarnings(as.numeric(as.character(df[, entity])))
    # We consider the entity numeric if all of its values can be converted to
    # numeric without introducing additional NAs.
    if (sum(is.na(entity.num)) == sum(is.na(df[, entity]))) {
      dropped.below.max <- entities.to.drop < max(entities.to.keep)
      dropped.above.min <- entities.to.drop > min(entities.to.keep)
      if (sum(dropped.below.max & dropped.above.min) > 0) {
        warning('Dropping numeric entities that are not at the edge of the ',
                'distribution. This may be not what you expected. For example ',
                'you may be tearing temporal holes in a panel data set.\nThe ',
                'remaining values are between ', min(entities.to.keep), ' and ',
                max(entities.to.keep), '.\nThe following values are being ',
                'dropped: ', paste(sort(entities.to.drop), collapse = ', '))
      }
    }
  }

  return(new.df)
}

plotAvailability <- function(df, ..., dimension2 = 'country.iso3') {
  vars <- DropMinusVars(...)
  tmp <- df[complete.cases(df[, vars]), ]
  par(mfrow = c(2, 1))
  plot(table(tmp$year), xlab = 'year', ylab = 'nobs')
  tab.dim2 <- table(tmp[, dimension2])
  plot(sort(tab.dim2, decreasing = TRUE),
       xlab = paste(dimension2, ' index'), ylab = 'nobs',
       ylim = c(0, max(tab.dim2) * 1.1))

  cat('NAs per variable:', fill = TRUE)
  print(sort(apply(tmp, 2, function(x) { sum(is.na(x)) }), decreasing = TRUE))
}

CreateCountrySelector <- function(exclude.iso3, negative = FALSE) {
  function(df) {
      df[(df$country.iso3 %in% exclude.iso3) != negative, ]
  }
}

# Manipulate collection of variable names
# -------------------------------------------------------------------------
DropMinusVars <- function(...) {
  all <- c(...)
  ind.minus <- grepl('^-', all)
  remaining <- all[!ind.minus & !(all %in% substring(all[ind.minus], 2))]
  remaining
}

DropInteractions <- function (..., include.interacted = TRUE,
                              return.interactions = FALSE) {
  all <- unique(c(...))
  all.but.int <- all[!grepl('(\\*|:)', all)]
  int.only <- all[grepl('(\\*|:)', all)]
  if (return.interactions) {
    int.only
  } else {
    if (include.interacted == FALSE) {
      all.but.int
    } else {
      individual.from.int <- unique(unlist(strsplit(int.only, '[:\\*]')))
      unique(c(all.but.int, individual.from.int))
    }
  }
}

DuplicateInteractions <- function(interactions) {
    # Takes a vector of interaction coefficients (e.g. var1:var2) and returns
    # all of their permutations. If the vector is named the coefficients are
    # assumed to be the names and the values are preserved.
    require(stringr)

    named.v <- !is.null(names(interactions))
    coefs <- if (named.v) names(interactions) else interactions
    vals  <- if (named.v) interactions        else NULL

    new.coefs <- c()
    new.vals <- c()
    for (i in 1:length(interactions)) {
        elements <- str_split(coefs[i], fixed(':'))[[1]]
        if (length(elements) == 1) {
            new.coefs <- c(new.coefs, coefs[i])
            new.vals <- c(new.vals, vals[i])
        } else {
            gr <- expand.grid(rep(list(elements), length(elements)))
            no.rep <- apply(gr, 1, function(x) {
                length(unique(x)) == length(x)
            })
            coefs.here <- apply(gr[no.rep, ], 1, function(x) {
                paste(x, collapse = ':')
            })
            new.coefs <- c(new.coefs, coefs.here)
            new.vals <- c(new.vals, rep(vals[i], length(coefs.here)))
        }
    }

    new        <- if (named.v) new.vals  else new.coefs
    names(new) <- if (named.v) new.coefs else NULL

    new
}

ApplyToPermutations <- function(x, split, fun,
                                drop.first = FALSE,
                                no.permutations = FALSE) {
    require(stringr)

    x.split <- str_split(x, fixed(split))
    x.new <- unlist(lapply(x.split, function(terms) {

        if (!no.permutations) {
            # We need all combinations of altered and unaltered terms
            index.df <- expand.grid(rep(list(c(FALSE, TRUE)), length(terms)))

            # Tweak: two terms, we want the second to be altered first
            if (nrow(index.df) == 4) {
              index.df[c(2, 3), ] <- index.df[c(3, 2), ]
            }

            # We not necessarily want the completely unaltered terms
            if (drop.first) {
                index.df <- index.df[-1, , drop = FALSE]
            }

            index.list <- split(index.df, seq(nrow(index.df)))

        } else {
            index.list <- as.list(TRUE)
        }


        lapply(index.list, function(index) {
            index <- unlist(index)
            modified <- terms
            modified[index] <- do.call(fun, list(modified[index]))
            paste(modified, collapse = split)
        })
    }))
    x.new
}

AddBetweenTerms <- function(interactions, ...) {
    nams.new <- ApplyToPermutations(names(interactions),
                                    ':',
                                    function(x) paste0(x, '.btw'),
                                    ...)
    vals.new <- ApplyToPermutations(interactions,
                                    ' $\\times$ ',
                                    function(x) paste0(x, ' (W)'),
                                    no.permutations = TRUE)
    vals.new <- ApplyToPermutations(vals.new,
                                    ' $\\times$ ',
                                    function(x) sub('(W)', '(B)', x, fixed = TRUE),
                                    ...)
    setNames(vals.new, nams.new)
}

# Manipulate variables
# -------------------------------------------------------------------------
center <- function(x)  x - mean(x, na.rm = TRUE)
# center <- function(x)  x - median(x, na.rm = TRUE)

CenterLogs <- function(df, exclude, include, only) {
  # Center all variables ending in '.l'
  require(stringr)

  if (missing(only)) {
    nm <- names(df)
    vars.to.center <- nm[str_detect(nm, '\\.l$')]
    if (!missing(include))  vars.to.center <- union(vars.to.center, include)
    if (!missing(exclude))  vars.to.center <- setdiff(vars.to.center, exclude)
  } else {
    vars.to.center <- only
  }

  df %>%
    mutate_each_(funs(center), vars.to.center)
}

# Regression methods
# -------------------------------------------------------------------------
MyFormula <- function(..., subs, dep.var = 'students') {
  paste0(dep.var, ' ~ ', paste(DropMinusVars(...), collapse = ' + '))
}

MyLm <- function(...,
                 data,
                 dep.var,
                 vars.subs = c(),
                 hc.robust = TRUE,
                 cluster = c(),
                 weights.col = c()) {
  require(lmtest)
  require(sandwich)
  require(multiwayvcov)

  # 1. Copy weight to new column
  # This is necessary, because the same variable might also be used as a
  # predictor. Some predictors are centered before the regression. Since weights
  # cannot be negative we have to prevent them from being centered.
  if (length(weights.col) > 0) {
    weights.col.internal <- 'MyWeights999'
    data[, weights.col.internal] <- data[, weights.col]
  } else {
    weights.col.internal <- NULL
  }

  # 2. Create subset
  subs <- MySubset(..., data = data,
                   vars.subs = c(vars.subs, weights.col.internal, cluster),
                   dep.var = dep.var)

  # 3. Copy weights to separate vector
  if (length(weights.col) > 0) {
    weights.data <- subs[, weights.col.internal]
  } else {
    weights.data <- NULL
  }

  # 4. Run the regression
  ind.vars <- DropMinusVars(...)
  est <- lm(as.formula(MyFormula(ind.vars, dep.var = dep.var)), data = subs,
            weights = weights.data)

  # 5. Remove TRUE from binary coefficients
  names(est$coefficients) <- gsub('TRUE', '', names(est$coefficients))

  # 6. Calculate ((cluster-)robust) standard errors and p values
  if (!hc.robust & length(cluster) == 0) {
    testresult <- coeftest(est)
  } else if (length(cluster) == 0) {
    testresult <- coeftest(est, vcovHC(est))
  } else {
    # cluster.vcov() needs the values of all cluster variables to be strings
    for (colname in cluster) {
        subs[, colname] <- as.character(subs[, colname])
    }
    testresult <- coeftest(est, cluster.vcov(est, cluster = subs[, cluster]))
  }

  # 7. Calculate custom GOF measurs
  gofs <- list()
  # 7a. Calculate numbers of entities
  count.gofs <- c('Num. institutes' = 'city',
                  'Num. countries' = 'country.iso3',
                  'Num. years' = 'year')
  gofs <- c(gofs, lapply(names(count.gofs), function(gof.name) {
    if (count.gofs[gof.name] %in% names(subs)) {
        list(name = gof.name,
             value = length(unique(subs[, count.gofs[gof.name]])),
             decimal = FALSE)
    }
  }))
  # 7b. Years
  year.min <- substr(min(as.numeric(as.character(subs[, 'year']))), 3, 4)
  year.max <- substr(max(as.numeric(as.character(subs[, 'year']))), 3, 4)
  gof.year <- list(name = 'Years',
                   value = as.numeric(paste0(year.min, '.', year.max)),
                   decimal = TRUE)
  gofs <- c(gofs, list(gof.year))

  list(est = est, se = testresult[, 2], pval = testresult[, 4], gof = gofs)
}

MyExamFe <- function(...,
                     data = default.data,
                     vars.subs = c(),
                     hc.robust = TRUE,
                     cluster = 'country.iso3') {
  MyLm(c(fe.yci, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'exams',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyExamLogFe <- function(...,
                        data      = default.data,
                        vars.subs = c(),
                        hc.robust = TRUE,
                        cluster   = 'country.iso3') {
  MyLm(c(fe.yci, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'exams.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyExamLogFeInstImmig <- function(...,
                        data      = default.data,
                        vars.subs = c(),
                        hc.robust = TRUE,
                        cluster   = 'country.iso3') {
  MyLm(c(fe.yc, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'mig.total.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyExamLogFeInstStud <- function(...,
                                 data      = default.data,
                                 vars.subs = c(),
                                 hc.robust = TRUE,
                                 cluster   = 'country.iso3') {
  MyLm(c(fe.yc, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'students.fy.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}


# Instrumental Variable Approach ___________________________________
MyFormulaIV <- function(..., instr, subs, dep.var = 'students') {
  paste0(dep.var, ' ~ ', paste(DropMinusVars(...), collapse = ' + '),
         ' | ', instr)
}

MyInstr <- function(...) {
  paste(DropMinusVars(...), collapse = ' + ')
}

MyIV <- function(...,
                 instr,
                 data,
                 dep.var,
                 vars.subs = c(),
                 hc.robust = TRUE,
                 cluster = c(),
                 weights.col = c()) {
  require(lmtest)
  require(sandwich)
  require(multiwayvcov)
  
  # 1. Copy weight to new column
  # This is necessary, because the same variable might also be used as a
  # predictor. Some predictors are centered before the regression. Since weights
  # cannot be negative we have to prevent them from being centered.
  if (length(weights.col) > 0) {
    weights.col.internal <- 'MyWeights999'
    data[, weights.col.internal] <- data[, weights.col]
  } else {
    weights.col.internal <- NULL
  }
  
  # 2. Create subset
  subs <- MySubset(..., data = data,
                   vars.subs = c(vars.subs, weights.col.internal, cluster),
                   dep.var = dep.var)
  
  # 3. Copy weights to separate vector
  if (length(weights.col) > 0) {
    weights.data <- subs[, weights.col.internal]
  } else {
    weights.data <- NULL
  }
  
  # 4. Run the regression
  ind.vars <- DropMinusVars(...)
  test <- MyFormulaIV(ind.vars, dep.var = dep.var, instr = MyInstr(instr))
  print(test)
  est <- ivreg(as.formula(#MyFormulaIV(ind.vars, dep.var = dep.var, instr = MyInstr(instr))
                        test), data = subs,
            weights = weights.data)
  
  # 5. Remove TRUE from binary coefficients
  names(est$coefficients) <- gsub('TRUE', '', names(est$coefficients))
  
  # 6. Calculate ((cluster-)robust) standard errors and p values
  if (!hc.robust & length(cluster) == 0) {
    testresult <- coeftest(est)
  } else if (length(cluster) == 0) {
    testresult <- coeftest(est, vcovHC(est))
  } else {
    # cluster.vcov() needs the values of all cluster variables to be strings
    for (colname in cluster) {
      subs[, colname] <- as.character(subs[, colname])
    }
    testresult <- coeftest(est, cluster.vcov(est, cluster = subs[, cluster]))
  }
  
  # 7. Calculate custom GOF measurs
  gofs <- list()
  # 7a. Calculate numbers of entities
  count.gofs <- c('Num. institutes' = 'city',
                  'Num. countries' = 'country.iso3',
                  'Num. years' = 'year')
  gofs <- c(gofs, lapply(names(count.gofs), function(gof.name) {
    if (count.gofs[gof.name] %in% names(subs)) {
      list(name = gof.name,
           value = length(unique(subs[, count.gofs[gof.name]])),
           decimal = FALSE)
    }
  }))
  # 7b. Years
  year.min <- substr(min(as.numeric(as.character(subs[, 'year']))), 3, 4)
  year.max <- substr(max(as.numeric(as.character(subs[, 'year']))), 3, 4)
  gof.year <- list(name = 'Years',
                   value = as.numeric(paste0(year.min, '.', year.max)),
                   decimal = TRUE)
  gofs <- c(gofs, list(gof.year))
  
  list(est = est, se = testresult[, 2], pval = testresult[, 4], gof = gofs)
}


MyExamLogIV <- function(...,
                        instr = c(),
                        data      = default.data,
                        vars.subs = c(),
                        hc.robust = TRUE,
                        cluster   = 'country.iso3') {
  MyIV(c(fe.yc, c(...)),
       instr     = c(fe.yci, instr),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'exams.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}
# _______________________________________________________________________

MyExamLogReFr0 <- function(...,
                        data      = default.data,
                        vars.subs = c(),
                        hc.robust = TRUE,
                        cluster   = 'country.iso3') {
  MyLm(c(fe.yr, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'exams.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyExamLogReFr1 <- function(...,
                           data      = default.data,
                           vars.subs = c(),
                           hc.robust = TRUE,
                           cluster   = 'country.iso3') {
  MyLm(c(fe.y, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'exams.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyRe <- function(..., dep.var, data, naive = FALSE, nsim = default.nsim) {
  require(lme4)
  require(stringr)
  require(magrittr)
  require(dplyr)
  require(boot)

  vars <- c(...)
  vars <- vars[!str_detect(vars, fixed('('))]
  vars <- DropInteractions(vars)

  if (nrow(data) != sum(complete.cases(data[, vars]))) {
    stop(paste0("Don't know how to handle missings, please provide ",
                "complete observations only."))
  }

  fmla <- MyFormula(..., dep.var = dep.var)

  set.seed(1)
  est <- lmer(fmla, data = data)

  if (naive) {
    # "Naive" standard errors, taken from texreg's extract.lme4

    se <- sqrt(diag(as.matrix(vcov(est, useScale = FALSE))))
    zval <- lme4::fixef(est) / se
    pval <- 2 * pnorm(abs(zval), lower.tail = FALSE)

  } else {
    # Bootstrapped standard errors

    set.seed(1)
    boot <- bootMer(est, fixef, nsim, parallel = 'multicore', ncpus = 4)

    boot.results <- data.frame(variable = names(fixef(est))) %>%
      # we want to preserve the order of the variables
      mutate(variable = factor(variable, levels = variable)) %>%
      mutate(variable.no = seq(1:nrow(.))) %>%
      # one line per variable - confidence level dyad
      group_by(variable, variable.no) %>%
      do(data.frame(confidence.level = c(0.90, 0.95, 0.99))) %>%
      ungroup() %>%
      # calculate confidence interval for each confidence level
      group_by(variable, variable.no, confidence.level) %>%
      do({
        ci <- boot.ci(boot, .$confidence.level, 'basic', .$variable.no)
        data.frame(lower       = ci$basic[4],
                   upper       = ci$basic[5],
                   significant = !(ci$basic[4] < 0 & ci$basic[5] > 0))
      }) %>%
      ungroup() %>%
      # calculate number of stars and pseudo p-value to feed to texreg
      group_by(variable, variable.no) %>%
      summarise(stars    = sum(significant),
                pseudo.p = min(1, (1 - confidence.level[significant]) - 0.00001,
                               na.rm = TRUE))

    se   <- apply(as.data.frame(boot), 2, sd, na.rm = TRUE)
    pval <- setNames(boot.results$pseudo.p, boot.results$variable)
  }

  list(est = est, se = se, pval = pval)
}

MyExamLogRe <- function(...,
                        data = default.data,
                        naive = FALSE,
                        nsim = default.nsim) {
  MyRe(..., dep.var = 'exams.l', data = data, naive = naive, nsim = nsim)
}

MyExamWFe <- function(...,
                      data = default.data,
                      vars.subs = c(),
                      cluster = 'country.iso3') {
  MyLm(c(fe.yci, c(...)),
       data = data,
       vars.subs = vars.subs,
       dep.var = 'exams',
       cluster = cluster,
       weights.col = 'mig.total.1k.f1')
}

MyGerFe <- function(...,
                    data = default.data,
                    vars.subs = c(),
                    hc.robust = TRUE,
                    cluster = c()) {
  MyLm(c(fe.yco, c(...)),
       data = data,
       vars.subs = vars.subs,
       dep.var = 'students.inger',
       hc.robust = hc.robust,
       cluster = cluster)
}

MyGerLogFe <- function(...,
                       data = default.data,
                       vars.subs = c(),
                       hc.robust = TRUE,
                       cluster = c('country.iso3')) {
  MyLm(c(fe.yco, c(...)),
       data = data,
       vars.subs = vars.subs,
       dep.var = 'students.inger.l',
       hc.robust = hc.robust,
       cluster = cluster)
}

MyGerLogIV <- function(...,
                        instr = c(),
                        data      = default.data,
                        vars.subs = c(),
                        hc.robust = TRUE,
                        cluster   = 'country.iso3') {
  MyIV(c(fe.yco, c(...)),
       instr     = c(fe.yco, instr),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'students.inger.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyGerLogFeInsttest <- function(...,
                                data      = default.data,
                                vars.subs = c(),
                                hc.robust = TRUE,
                                cluster   = 'country.iso3') {
  MyLm(c(fe.yco, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'mig.total.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyGerLogFeInststud <- function(...,
                               data      = default.data,
                               vars.subs = c(),
                               hc.robust = TRUE,
                               cluster   = 'country.iso3') {
  MyLm(c(fe.yco, c(...)),
       data      = data,
       vars.subs = vars.subs,
       dep.var   = 'students.fy.l',
       hc.robust = hc.robust,
       cluster   = cluster)
}

MyGerLogReNew <- function(...,
                       data = default.data,
                       vars.subs = c(),
                       hc.robust = TRUE,
                       cluster = c('country.iso3')) {
  MyLm(c(fe.yr, c(...)),
       data = data,
       vars.subs = vars.subs,
       dep.var = 'students.inger.l',
       hc.robust = hc.robust,
       cluster = cluster)
}

MyGerLogReNewFr1 <- function(...,
                          data = default.data,
                          vars.subs = c(),
                          hc.robust = TRUE,
                          cluster = c('country.iso3')) {
  MyLm(c(fe.y, c(...)),
       data = data,
       vars.subs = vars.subs,
       dep.var = 'students.inger.l',
       hc.robust = hc.robust,
       cluster = cluster)
}

MyGerLogRe <- function(..., data = default.data) {
    MyRe(..., dep.var = 'students.inger.l', data = data)
}

MyGerWFe <- function(...,
                    data = default.data,
                    vars.subs = c(),
                    hc.robust = TRUE,
                    cluster = c('country.iso3')) {
  MyLm(c(fe.yco, c(...)),
       data = data,
       vars.subs = vars.subs,
       dep.var = 'students.inger',
       hc.robust = hc.robust,
       cluster = cluster,
       weights.col = 'mig.total.1k.l1')
}

# Save all estimation results in a list to separate files
# This is helpful, when we want to combine the results from estimations that are
# run in different files.
saveEst <- function(l, fn.template) {
  for (name in names(l)) {
    saveRDS(l[[name]], str_replace(fn.template, fixed('NAME'), name))
  }
}

# Reporting
# -------------------------------------------------------------------------
UnpackModelList <- function(model.list) {

  # If the model list contains list instead of model objects, we assume that
  # these lists have three components: (1) a model object est, (2) a vector of
  # standard errors se, and (3) a vector of p-values pval.
  models <- list()
  ses <- list()
  pvals <- list()
  gofs <- list()
  for (model.name in names(model.list)) {
    model <- model.list[[model.name]]
    if (is(model)[1] == 'list') {
      models[[model.name]] <- model[['est']]
      ses[[model.name]] <- model[['se']]
      pvals[[model.name]] <- model[['pval']]
      gofs[[model.name]] <- model[['gof']]
    } else {
      models[[model.name]] <- model
      ses[[model.name]] <- summary(model)$coefficients[, 2]
      pvals[[model.name]] <- summary(model)$coefficients[, 4]
      gofs[[model.name]] <- list()
    }
  }
  if (length(ses) == 0) ses <- 0
  if (length(pvals) == 0) pvals <- 0

  list(models = models, ses = ses, pvals = pvals, gofs = gofs)
}

MyTexRegOld <- function(model.list, ...,
                        sort.order = c(),
                        replacements = c(),
                        drop.lines = c(),
                        font.size = 'scriptsize',
                        font.size.no = c(),
                        omit.coef = '(year|country|city\\.de|Intercept)',
                        file = NULL,
                        float.pos = 'htb') {
  # Execute texreg and post-process its output
  # replacements: A vector of strings to be replaced. Vector names are replaced
  #   by vector values.
  require(texreg)

  info <- UnpackModelList(model.list)

  # Add custom GOF measures
  for (mn in names(info[['models']])) {
    info[['models']][[mn]] <- AddCustomGofs(info[['models']][[mn]],
                                            info[['gofs']][[mn]])
  }

  # We will operate on the output of a texreg call. Unfortunately, there is no
  # proper way to prevent texreg from printing its output as well. So we have
  # to capture it and throw it away.
  zzz <- capture.output(
    tr <- texreg(info[['models']],
                 override.se = info[['ses']],
                 override.pval = info[['pvals']],
                 omit.coef = omit.coef,
                 float.pos = float.pos,
                 fontsize = font.size,
                 booktabs = TRUE, dcolumn = TRUE, use.packages = FALSE,
                 stars = c(0.01, 0.05, 0.1),
                 ...)
  )
  rm(zzz)

  tr <- CustomizeTable(tr, 3,
                       sort.order = sort.order,
                       replacements = replacements,
                       drop.lines = drop.lines)

  if (length(font.size.no) > 0) {
    tr <- str_replace(tr, fixed(paste0('\\begin{', font.size, '}')),
                      paste0('{\\fontsize{', font.size.no[1], '}{',
                             font.size.no[1], '}\\selectfont'))
    tr <- str_replace(tr, fixed(paste0('\\end{', font.size, '}')), '}')
  }

  if (is.null(file)) {
      cat(tr)
  } else {
    cat(tr, file = file)
  }
}

AddCustomGofs <- function(model, gofs) {
    tr <- texreg::extract(model, include.rmse = FALSE)
    for (gof in gofs) {
        tr@gof <- c(tr@gof, gof[['value']])
        tr@gof.names <- c(tr@gof.names, gof[['name']])
        tr@gof.decimal <- c(tr@gof.decimal, gof[['decimal']])
    }
    tr
}

MyScreenReg <- function(model.list, names.map, ...) {
    info <- UnpackModelList(model.list)

    screenreg(info[['models']],
              custom.coef.map = names.map,
              ...,
              override.se     = info[['ses']],
              override.pval   = info[['pvals']],
              naive           = TRUE,
              stars           = c(0.01, 0.05, 0.1))
}

MyTexReg <- function(model.list, names.map, ...) {
    info <- UnpackModelList(model.list)

    texreg(info[['models']],
           custom.coef.map = names.map,
           ...,
           override.se     = info[['ses']],
           override.pval   = info[['pvals']],
           table           = FALSE,
           booktabs        = TRUE,
           dcolumn         = TRUE,
           use.packages    = FALSE,
           naive           = TRUE,
           stars           = c(0.01, 0.05, 0.1),
           include.rmse    = FALSE,
           include.aic     = FALSE,
           include.bic     = FALSE,
           include.loglik  = FALSE)
}

