# setwd()

source('00-libraries.R')

# Prep
source('prep_abr.R')
source('estimation_helpers.R')
source('sets_of_variables.R')

# Log-log estimation
# -------------------------------------------------------------------------
default.data <- CenterLogs(d)
source('subsetting_functions.R')

country.popstats <- d %>%
  filter(!(jr.case1 | jr.case2 | jr.case3 | jr.case4 | jr.case5)) %>%
  group_by(country.iso3, city) %>%
  summarise(is.capital = first(is.capital),
            pop        = round(mean(pop, na.rm = TRUE)/1000000, 2)) %>%
  summarise(cities      = n(),
            noncapitals = sum(!is.capital),
            pop         = first(pop)) %>%
  mutate(has.noncapital = noncapitals > 0,
         twocities      = cities > 1)
pop.med <- median(country.popstats$pop)
countries.large <- filter(country.popstats, pop > pop.med)$country.iso3
countries.small <- filter(country.popstats, pop <= pop.med)$country.iso3
CountryL <- . %>% filter(country.iso3 %in% countries.large)
CountryS <- . %>% filter(country.iso3 %in% countries.small)

country.ldist <- d %>%
  filter(!(jr.case1 | jr.case2 | jr.case3 | jr.case4 | jr.case5)) %>%
  group_by(country.iso3) %>%
  summarise(ling.dist = first(ling.dist)) %>%
  spread(country.iso3, ling.dist) %>%
  mutate(BEL = (NLD + FRA) / 2) %>%
  gather(country.iso3, ling.dist)
pdf(file = 'fig-histogram_lingdist.pdf', width = 5, height = 5)
country.ldist %>%
  drop_na(ling.dist) %>%
  arrange(ling.dist) %>%
  {hist(.$ling.dist,
        main = 'Histogram: Linguistic Distance',
        xlab = 'Linguistic Distance')}
dev.off()
countries.lingd <- filter(country.ldist, ling.dist > 0.75)$country.iso3
countries.lingc <- filter(country.ldist, ling.dist <= 0.75)$country.iso3
length(countries.lingc)
CountryLingD <- . %>% filter(country.iso3 %in% countries.lingd)
CountryLingC <- . %>% filter(country.iso3 %in% countries.lingc)

### Do not exclude Madrid due to joint reporting
d$jr.case1[d$city=="Madrid"] <- FALSE

# Man <- function(x)  x
FilterJr <- . %>%
  filter(!jr.case1) %>%
  filter(!jr.case2) %>%
  filter(!jr.case3) %>%
  filter(!jr.case4) %>%
  filter(!jr.case5)

d %>%
  FilterJr() %>%
  saveRDS('d-estimations_abr_fe.rds')

d %>%
  Li06() %>%
  Eu1()

Man <- . %>%
  FilterJr() %>%
  CenterLogs()

# IV-Approach ------------------------------------------------------------------
library(AER)

# Read in IV data
dIV <- readRDS('dIV.rds')

# First stage (EU and Schengen)
e <- list()
e[['fr1s']]       <- MyExamLogFeInstImmig(t.iv.abr.inst.fr,    data = Man(Fr1(dIV)))
e[['nonfr1s']]    <- MyExamLogFeInstImmig(t.iv.abr.inst.nonfr, data = Man(Fr0(dIV)))

# Second stage (EU and Schengen)
e[['fr2s']]       <- MyExamLogIV(t.iv.abr.ind, instr = t.iv.abr.inst.fr,    data = Man(Fr1(dIV)))
e[['nonfr2s']]    <- MyExamLogIV(t.iv.abr.ind, instr = t.iv.abr.inst.nonfr, data = Man(Fr0(dIV)))

e[['frFE']]       <- MyExamLogFe(t.iv.abr.ind, data = Man(Fr1(dIV)))
e[['nonfrFE']]    <- MyExamLogFe(t.iv.abr.ind, data = Man(Fr0(dIV)))

MyScreenReg(e, as.list(vl.bea))

saveEst(e, 'tmp_output/est-abr_log_IV_NAME.rds')

# Test for Instrument Validity
# Free Move
fr1s          <- MyExamLogFeInstImmig(t.iv.abr.inst.fr, data = Man(Fr1(dIV)))$est
fr1sresiduals <- fr1s$residuals
fr1sres       <- MyExamLogFeInstImmig(t.iv.abr.instres, data = Man(Fr1(dIV)))$est

instrFtestfr <- waldtest(fr1s, fr1sres)
print(instrFtestfr)
# we can clearly reject the null hypothesis that the instruments are irrelevant

# Restricted Move
nonfr1s          <- MyExamLogFeInstImmig(t.iv.abr.inst.nonfr, data = Man(Fr0(dIV)))$est
nonfr1sresiduals <- nonfr1s$residuals
nonfr1sres       <- MyExamLogFeInstImmig(t.iv.abr.instres,    data = Man(Fr0(dIV)))$est

instrFtestnonfr <- waldtest(nonfr1s, nonfr1sres)
print(instrFtestnonfr)
# we can clearly reject the null hypothesis that the instruments are irrelevant

# ------------------------------------------------------------------------------

# Table 3: Abroad Schengen OR EU, stepwise
e[['fr1']]    <- MyExamLogFe(t.abr.1, data = Man(Fr1(d)))
e[['nonfr1']] <- MyExamLogFe(t.abr.1, data = Man(Fr0(d)))
e[['fr2']]    <- MyExamLogFe(t.abr.2, data = Man(Fr1(d)))
e[['nonfr2']] <- MyExamLogFe(t.abr.2, data = Man(Fr0(d)))
e[['fr']]    <- MyExamLogFe(t.abr.main, data = Man(Fr1(d)))
e[['nonfr']] <- MyExamLogFe(t.abr.main, data = Man(Fr0(d)))

# FDI instead of trade:
e[['fdi']]   <- MyExamLogFe(t.abr.main.fdi)

e[['highi99']] <- MyExamLogFe(t.abr.main, data = Hi99(d))
e[['highi06']] <- MyExamLogFe(t.abr.main, data = Hi06(d))
e[['lowi99']]  <- MyExamLogFe(t.abr.main, data = Li99(d))
e[['lowi06']]  <- MyExamLogFe(t.abr.main, data = Li06(d))

# Robustness check: large cities
e[['cap']]    <- MyExamLogFe(t.abr.main, data = Cap(d))
e[['capeu']]  <- MyExamLogFe(t.abr.main, data = Man(Eu1(Cap(d))))
e[['capneu']] <- MyExamLogFe(t.abr.main, data = Man(Eu0(Cap(d))))
e[['ncapeu']]  <- MyExamLogFe(t.abr.main, data = Man(Eu1(NoCap(d))))
e[['ncapneu']] <- MyExamLogFe(t.abr.main, data = Man(Eu0(NoCap(d))))

# Robustness check: big and small countries
e[['big']]      <- MyExamLogFe(t.abr.main, data = Man(CountryL(d)))
e[['small']]    <- MyExamLogFe(t.abr.main, data = Man(CountryS(d)))
e[['bigeu']]    <- MyExamLogFe(t.abr.main, data = Man(Eu1(CountryL(d))))
e[['bigneu']]   <- MyExamLogFe(t.abr.main, data = Man(Eu0(CountryL(d))))
e[['smalleu']]  <- MyExamLogFe(t.abr.main, data = Man(Eu1(CountryS(d))))
e[['smallneu']] <- MyExamLogFe(t.abr.main, data = Man(Eu0(CountryS(d))))

# Table 7: Robustness check: high and low income countries, Schengen and non-Schengen
e[['hifr']]   <- MyExamLogFe(t.abr.main, data = Man(Fr1(Hi06(d))))
e[['hinfr']]  <- MyExamLogFe(t.abr.main, data = Man(Fr0(Hi06(d))))
e[['lifr']]   <- MyExamLogFe(t.abr.main, data = Man(Fr1(Li06(d))))
e[['linfr']]  <- MyExamLogFe(t.abr.main, data = Man(Fr0(Li06(d))))

d %>%
  Li06() %>%
  Eu0() %>%
  Man() %>%
  select(country.iso3, city, year) %>%
  summarise_all(funs(length(unique(.))))

# Robustness check: high and low linguistic distance
e[['lclose']] <- MyExamLogFe(t.abr.main, data = Man(CountryLingC(d)))
e[['ldist']]  <- MyExamLogFe(t.abr.main, data = Man(CountryLingD(d)))
e[['lceu']]   <- MyExamLogFe(t.abr.main, data = Man(Eu1(CountryLingC(d))))
e[['lcneu']]  <- MyExamLogFe(t.abr.main, data = Man(Eu0(CountryLingC(d))))
e[['ldeu']]   <- MyExamLogFe(t.abr.main, data = Man(Eu1(CountryLingD(d))))
e[['ldneu']]  <- MyExamLogFe(t.abr.main, data = Man(Eu0(CountryLingD(d))))

# Table 6: Robustness check: Largest cities and other - Schengen OR EU
e[['big']]    <- MyExamLogFe(t.abr.main, data = Big(d))
e[['bigfr']]  <- MyExamLogFe(t.abr.main, data = Man(Fr1(Big(d))))
e[['bignfr']] <- MyExamLogFe(t.abr.main, data = Man(Fr0(Big(d))))
e[['nbigfr']]  <- MyExamLogFe(t.abr.main, data = Man(Fr1(NoBig(d))))
e[['nbignfr']] <- MyExamLogFe(t.abr.main, data = Man(Fr0(NoBig(d))))
e[['ncapbigfr']]  <- MyExamLogFe(t.abr.main, data = Man(Fr1(Big(NoCap(d)))))
e[['ncapbignfr']] <- MyExamLogFe(t.abr.main, data = Man(Fr0(Big(NoCap(d)))))

MyScreenReg(e, as.list(vl.bea))

saveEst(e, 'tmp_output/est-abr_log_fe_NAME.rds')


# Non-FE Estimations 
# ______________________________________________________________________________
# Non-FE: Abroad Free and Restricted Move
dFr1 <- readRDS('dFr1.rds')
dFr0 <- readRDS('dFr0.rds')

e <- list()
e[['frRE']]    <- MyExamLogReFr1(t.abr.fr1, data = Man(dFr1))
e[['nonfrRE']] <- MyExamLogReFr0(t.abr.fr0, data = Man(dFr0))

# FE: Abroad Free and Restricted Move
e[['frFE']]    <- MyExamLogFe(t.abr.main, data = Man(dFr1))
e[['nonfrFE']] <- MyExamLogFe(t.abr.main, data = Man(dFr0))


MyScreenReg(e, as.list(vl.bea))

saveEst(e, 'tmp_output/est-abr_log_re_NAME.rds')

# ______________________________________________________________________________

# Ad-hoc plot of mig.total and mig.stock
d %>%
  Eu0() %>%
  group_by(year, country.iso3) %>%
  summarise(mig.total = first(mig.total),
            mig.stock = first(mig.stock)) %>%
  group_by(country.iso3) %>%
  summarise(mig.total = mean(mig.total, na.rm = TRUE),
            mig.stock = mean(mig.stock, na.rm = TRUE)) %>%
  arrange(desc(mig.stock))
  {hist(.$mig.stock, breaks = 100)}
d %>%
  Eu1() %>%
  Man() %>%
  select(country.iso3, city, year, exams, pop) %>%
  print(n = 500)
# ---------------------------------------
# Observations per country
d %>%
  group_by(year, country.iso3) %>%
  summarise(n = n()) %>%
  group_by(country.iso3) %>%
  summarise(n = n()) %>%
  arrange(n) %>%
  as.data.frame()

# Interaction Plot
# -------------------------------------------------------------------------
library(ggplot2)
library(scales)

mig.stock.means <- d %>%
  Eu0() %>%
  Man() %>%
  group_by(country.iso3, year) %>%
  summarise(mig.stock = first(mig.stock)) %>%
  summarise(mig.stock = mean(mig.stock)) %>%
  filter(mig.stock <= 300000)

mig.stock.means %>%
  print(n = 100)

cfs <- coef(e[['noneu']]$est)
rng <- range(Man(Eu0(d))[['mig.stock.l']])
tibble(x         = seq(rng[1], rng[2], length.out = 500),
       x.uncl    = exp(mean(Eu0(d)$mig.stock.l) + x),
       effect    = cfs[['mig.total.l']] + cfs[['mig.total.l:mig.stock.l']] * x) %>%
  filter(x.uncl <= 300000) %>%
  ggplot(aes(x = x.uncl, y = effect)) +
    geom_line() +
    geom_rug(aes(x = mig.stock),
             data = mig.stock.means,
             alpha = 0.2,
             inherit.aes = FALSE) +
    scale_x_continuous(labels = comma) +
    theme_bw() +
    xlab('Migrant Stock') +
    ylab('Migration Elasticity of Exam Participation')
ggsave('fig-interaction.pdf', width = 7, height = 4)


Man(Eu0(d)) %>%
  filter(exams.l < 0)

# Diagnostic plots
pdf('fig-abr-log-main-sens.pdf', width = 5, height = 5)
par(mfrow = c(2, 2), mar = c(3, 2, 1.5, 1))
plot(e[['eu']]$est)
dev.off()

d %>%
  Eu0() %>%
  filter(row_number() %in% c(372:374, 497:499, 1151:1153)) %>%
  select(country.iso3, year, city, exams, population.l, pop.city.l,
         mig.total.l, mig.stock.l, students.fy.l, trade.l, gdp.pc.l)

# Country-sensitivity
source('../Rhelpers/MyEntitySensitivity.R')
res <- MyEntitySensitivity(c(t.abr.main),
                           Man(NoCap(Eu0(d))),
                           MyExamLogFe,
                           'country.iso3',
                           'city|year')

library(ggplot2)

# Change of immigration and stocks over years
dman <- Man(d) # Filter data used in estimations

# Immigration
byyear <- group_by(dman, year) 
immigration_dev <- summarize(byyear, immig = mean(mig.total, na.rm = TRUE))

ggplot(data = immigration_dev) +
  geom_bar(mapping = aes(x = year, y = immig), stat = "identity")
ggsave("graphs/immigration_dev_abr.png")

# Migration stocks
byyear <- group_by(dman, year) 
migstock_dev <- summarize(byyear, migstock = mean(mig.stock, na.rm = TRUE))

ggplot(data=migstock_dev, mapping = aes(x = year, y = migstock)) +
  geom_point() +
  geom_abline()

ggplot(data = migstock_dev) +
  geom_bar(mapping = aes(x = year, y = migstock), stat = "identity")
ggsave("graphs/migstock_dev_abr.png")

# Student migration
byyear <- group_by(dman, year) 
ffystud_dev <- summarize(byyear, ffystud = mean(students.fy, na.rm = TRUE))

ggplot(data = ffystud_dev) +
  geom_bar(mapping = aes(x = year, y = ffystud), stat = "identity")
ggsave("graphs/ffystud_dev_abr.png")

## Correlation between Immigration and Migrants Stocks
# Whole sample (d)
ggplot(data = dman) +
  geom_point(mapping = aes(x = mig.total.l, y = mig.stock.l)) +
  geom_abline(color = "blue")
ggsave("graphs/corr_mig_total_stock_abr.png")

cor(dman$mig.total.l, dman$mig.stock.l, method = c("spearman"))

# Free Move (d)
dmanFr1 <- Fr1(dman)

ggplot(data = dmanFr1) +
  geom_point(mapping = aes(x = mig.total.l, y = mig.stock.l)) +
  geom_abline(color = "blue")
ggsave("graphs/corr_mig_total_stock_abr_freemove.png")
cor(dmanFr1$mig.total.l, dmanFr1$mig.stock.l, method = c("spearman"))

# Restr. Move (d)
dmanFr0 <- Fr0(dman)

ggplot(data = dmanFr0) +
  geom_point(mapping = aes(x = mig.total.l, y = mig.stock.l)) +
  geom_abline(color = "blue")
ggsave("graphs/corr_mig_total_stock_abr_restmove.png")

cor(dmanFr0$mig.total.l, dmanFr0$mig.stock.l, method = c("spearman"))
# ______________________________________________________________________________

d %>%
  NoCap() %>%
  Man() %>%
  filter(country.iso3 == 'ESP') %>%
  select(country.iso3, year, city, exams, pop, one_of(str_replace(t.abr.main, '\\.l', '')))
  ggplot(aes(x = students.fy, y = exams, label = year)) +
    geom_text()

d %>%
  NoCap() %>%
  Man() %>%
  filter(exams < 10)


# Are NGA and CIV countries with negligible migration?
d %>%
  group_by(country.iso3) %>%
  summarise()

# Dataset statistics
dIV %>%
  Man() %>%
  #Li06() %>%
  Fr1() %>%
  summarise(n = n(),
            cities = length(unique(city)),
            countries = length(unique(country.iso3)),
            years = length(unique(year)))

